@@ -6,8 +6,9 @@ use strict;
 use Lemonldap::NG::Common::Regexp;
 use Lemonldap::NG::Handler::Main;
 use Lemonldap::NG::Common::Util qw(getSameSite);
+use URI;
 
-our $VERSION = '2.0.12';
+our $VERSION = '2.18.0';
 
 ## @method hashref tests(hashref conf)
 # Return a hash ref where keys are the names of the tests and values
@@ -44,10 +45,18 @@ sub tests {
 
         # Check if portal URL is well formated
         portalURL => sub {
+            my $url = $conf->{portal};
 
-            # Append or remove trailing ending slashes
+            # Append or remove trailing slashes
             $conf->{portal} =~ s%/*$%/%;
-            return 1;
+            return (
+                1,
+                (
+                    ( $url =~ m%/$% )
+                    ? ''
+                    : "Portal URL should end with a /"
+                )
+            );
         },
 
         # Check if virtual hosts are in the domain
@@ -348,22 +357,19 @@ sub tests {
             my $res = 1;
             my %entityIds;
             foreach my $idpId ( keys %{ $conf->{samlIDPMetaDataXML} } ) {
-                unless (
+                if (
                     $conf->{samlIDPMetaDataXML}->{$idpId}->{samlIDPMetaDataXML}
                     =~ /entityID=(['"])(.+?)\1/si )
                 {
-                    push @msg, "$idpId SAML metadata has no EntityID";
-                    $res = 0;
-                    next;
+                    my $eid = $2;
+                    if ( defined $entityIds{$eid} ) {
+                        push @msg,
+"$idpId and $entityIds{$eid} have the same SAML EntityID";
+                        $res = 0;
+                        next;
+                    }
+                    $entityIds{$eid} = $idpId;
                 }
-                my $eid = $2;
-                if ( defined $entityIds{$eid} ) {
-                    push @msg,
-                      "$idpId and $entityIds{$eid} have the same SAML EntityID";
-                    $res = 0;
-                    next;
-                }
-                $entityIds{$eid} = $idpId;
             }
             return ( $res, join( ', ', @msg ) );
         },
@@ -375,22 +381,19 @@ sub tests {
             my $res = 1;
             my %entityIds;
             foreach my $spId ( keys %{ $conf->{samlSPMetaDataXML} } ) {
-                unless (
+                if (
                     $conf->{samlSPMetaDataXML}->{$spId}->{samlSPMetaDataXML} =~
                     /entityID=(['"])(.+?)\1/si )
                 {
-                    push @msg, "$spId SAML metadata has no EntityID";
-                    $res = 0;
-                    next;
+                    my $eid = $2;
+                    if ( defined $entityIds{$eid} ) {
+                        push @msg,
+"$spId and $entityIds{$eid} have the same SAML EntityID";
+                        $res = 0;
+                        next;
+                    }
+                    $entityIds{$eid} = $spId;
                 }
-                my $eid = $2;
-                if ( defined $entityIds{$eid} ) {
-                    push @msg,
-                      "$spId and $entityIds{$eid} have the same SAML EntityID";
-                    $res = 0;
-                    next;
-                }
-                $entityIds{$eid} = $spId;
             }
             return ( $res, join( ', ', @msg ) );
         },
@@ -542,6 +545,22 @@ sub tests {
                 ) if ($@);
             }
 
+            # Use WebAuthn
+            if ( $conf->{webauthn2fActivation} ) {
+                eval "use Authen::WebAuthn";
+                return ( 1,
+                    "Authen::WebAuthn module is required to enable WebAuthn" )
+                  if ($@);
+            }
+
+            # WebAuthn requires https://
+            if ( $conf->{webauthn2fActivation} ) {
+                my $portal_uri = URI->new( $conf->{portal} );
+                unless ( $portal_uri->scheme eq "https" ) {
+                    return ( 1, "WebAuthn requires HTTPS" );
+                }
+            }
+
             # Use Yubikey
             if ( $conf->{yubikey2fActivation} ) {
                 eval "use Auth::Yubikey_WebClient";
@@ -622,7 +641,7 @@ sub tests {
 
             my $msg = '';
             my $ok  = 0;
-            foreach (qw(u totp yubikey)) {
+            foreach (qw(u totp yubikey webauthn)) {
                 $ok ||= $conf->{ $_ . '2fActivation' }
                   && $conf->{ $_ . '2fSelfRegistration' };
                 last if ($ok);
@@ -663,8 +682,10 @@ sub tests {
             return 1 unless ( defined $conf->{issuersTimeout} );
             return ( 0, "Issuers token TTL must be higher than 30s" )
               unless ( $conf->{issuersTimeout} > 30 );
-            return ( 1, "Issuers token TTL should not be higher than 2mn" )
-              if ( $conf->{issuersTimeout} > 120 );
+
+            # because of issue #2186
+            return ( 1, "Issuers token TTL should not be higher than 10mn" )
+              if ( $conf->{issuersTimeout} > 600 );
             return 1;
         },
 
@@ -738,8 +759,8 @@ sub tests {
             return ( 1,
                 "Impersonation and ContextSwitching are simultaneously enabled"
               )
-              if ( $conf->{impersonationRule}
-                && $conf->{contextSwitchingRule} );
+              if (  $conf->{impersonationRule}
+                and $conf->{contextSwitchingRule} );
             return 1;
         },
 
@@ -813,6 +834,16 @@ sub tests {
             return ( $res, join( ', ', @msg ) );
         },
 
+        # Test if SAML private and public keys signature keys are set
+        oidcSecretKeys => sub {
+            return 1 unless ( $conf->{issuerDBOpenIDConnectActivation} );
+            return ( 1,
+                'OIDC service private and public keys signature should be set' )
+              unless ( $conf->{oidcServicePrivateKeySig}
+                && $conf->{oidcServicePublicKeySig} );
+            return 1;
+        },
+
         # RS* OIDC algs require a signing key
         oidcRPNeedRSAKey => sub {
             return 1
@@ -842,6 +873,33 @@ sub tests {
             return 1;
         },
 
+        # Public OIDC clients require a public key algorithm
+        oidcRPPublicNeedPubAlg => sub {
+            return 1
+              unless ( $conf->{oidcRPMetaDataOptions}
+                and %{ $conf->{oidcRPMetaDataOptions} || {} } );
+            my @clients;
+            for ( keys %{ $conf->{oidcRPMetaDataOptions} || {} } ) {
+                if ( $conf->{oidcRPMetaDataOptions}->{$_}
+                    ->{oidcRPMetaDataOptionsPublic}
+                    and $conf->{oidcRPMetaDataOptions}->{$_}
+                    ->{oidcRPMetaDataOptionsIDTokenSignAlg} =~ /^HS/ )
+                {
+                    push @clients, $_;
+                }
+            }
+            if (@clients) {
+                my $msg =
+                    join( ", ", @clients )
+                  . ": public clients should use a public key algorithm"
+                  . " for ID token signature";
+                return 1, $msg;
+            }
+            else {
+                return 1;
+            }
+        },
+
         # OIDC RP Client ID must exist and be unique
         oidcRPClientIdUniqueness => sub {
             return 1
@@ -853,7 +911,8 @@ sub tests {
             foreach
               my $clientConfKey ( keys %{ $conf->{oidcRPMetaDataOptions} } )
             {
-                my $clientId = $conf->{oidcRPMetaDataOptions}->{$clientConfKey}
+                my $clientId =
+                  $conf->{oidcRPMetaDataOptions}->{$clientConfKey}
                   ->{oidcRPMetaDataOptionsClientID};
                 unless ($clientId) {
                     push @msg,
@@ -883,25 +942,31 @@ sub tests {
             my %casUrl;
             foreach my $casConfKey ( keys %{ $conf->{casAppMetaDataOptions} } )
             {
-                my $appUrl =
-                  $conf->{casAppMetaDataOptions}->{$casConfKey}
-                  ->{casAppMetaDataOptionsService}
-                  || "";
-                $appUrl =~ m#^(https?://[^/]+)(/.*)?$#;
-                my $appHost = $1;
-                unless ($appHost) {
-                    push @msg, "$casConfKey CAS Application has no Service URL";
-                    $res = 0;
-                    next;
-                }
+                for my $appUrl (
+                    split(
+                        /\s+/,
+                        $conf->{casAppMetaDataOptions}->{$casConfKey}
+                          ->{casAppMetaDataOptionsService}
+                    )
+                  )
+                {
+                    $appUrl ||= "";
+                    my ($appHost) = $appUrl =~ m#^(https?://[^/]+)(/.*)?$#;
+                    unless ($appHost) {
+                        push @msg,
+                          "$casConfKey CAS Application has no Service URL";
+                        $res = 0;
+                        next;
+                    }
 
-                if ( defined $casUrl{$appUrl} ) {
-                    push @msg,
+                    if ( defined $casUrl{$appUrl} ) {
+                        push @msg,
 "$casConfKey and $casUrl{$appUrl} have the same Service URL";
-                    $res = 0;
-                    next;
+                        $res = 0;
+                        next;
+                    }
+                    $casUrl{$appUrl} = $casConfKey;
                 }
-                $casUrl{$appUrl} = $casConfKey;
             }
             return ( $res, join( ', ', @msg ) );
         },
@@ -911,7 +976,9 @@ sub tests {
             return 1 unless ( $conf->{sfRemovedMsgRule} );
             return ( 1,
 'Notification system must be enabled to display a notification if a SF is removed'
-            ) if ( $conf->{sfRemovedUseNotif} and not $conf->{notification} );
+              )
+              if ( $conf->{sfRemovedUseNotif}
+                and not $conf->{notification} );
             return 1;
         },
 
@@ -928,7 +995,8 @@ sub tests {
         # Same with SameSite=(auto) and SAML issuer in use
         SameSiteNoneWithSecure => sub {
             return ( -1, 'SameSite value = None requires the secured flag' )
-              if ( getSameSite($conf) eq 'None' and !$conf->{securedCookie} );
+              if ( getSameSite($conf) eq 'None'
+                and !$conf->{securedCookie} );
             return 1;
         },
 
@@ -952,7 +1020,7 @@ sub tests {
             {
                 my $hasPwdBE = 0;
                 foreach ( keys %{ $conf->{authChoiceModules} } ) {
-                    my @mods = split /[;\|]/, $conf->{authChoiceModules}->{$_};
+                    my @mods = split /;\s*/, $conf->{authChoiceModules}->{$_};
                     $hasPwdBE ||= 1 unless $mods[2] eq 'Null';
                 }
                 return ( -1,
@@ -1024,7 +1092,101 @@ sub tests {
               if ( $conf->{authentication} eq 'Choice'
                 and scalar keys %{ $conf->{authChoiceModules} } == 0 );
             return 1;
-        }
+        },
+
+        # Internal portal URL must be defined with Proxy authentication
+        authProxy => sub {
+            return ( 0,
+                'Proxy authentication enabled without internal portal URL' )
+              if ( $conf->{authentication} eq 'Proxy'
+                and !$conf->{proxyAuthService} );
+            return 1;
+        },
+
+# Warn if Impersonation and proxyAuthServiceImpersonation are simultaneously enabled
+        impersonationProxy => sub {
+            return ( -1,
+'Impersonation and internal portal Impersonation are simultaneously enabled'
+              )
+              if (  $conf->{impersonationRule}
+                and $conf->{proxyAuthServiceImpersonation} );
+            return 1;
+        },
+
+        # CheckDevOps requires Safe jail
+        checkDevOpsWithSafeJail => sub {
+            return ( 0, 'Safe jail must be enabled with CheckDevOps plugin' )
+              if ( $conf->{checkDevOps}
+                and !$conf->{useSafeJail} );
+            return 1;
+        },
+
+        # Work around for #1740
+        corruptApplicationConfig => sub {
+            for my $cat ( keys %{ $conf->{applicationList} || {} } ) {
+                if ( ref( $conf->{applicationList}->{$cat} ) eq "HASH" ) {
+                    for my $app (
+                        keys %{ $conf->{applicationList}->{$cat} || {} } )
+                    {
+                        if (
+                            ref( $conf->{applicationList}->{$cat}->{$app} ) eq
+                            "HASH"
+                            and
+                            $conf->{applicationList}->{$cat}->{$app}->{type} eq
+                            "menuApp" )
+                        {
+                            return ( 0,
+                                    'Error saving application list.'
+                                  . ' Reload the manager and try again' );
+                        }
+                    }
+                }
+            }
+            return 1;
+        },
+
+        # OIDC Signature and Encryption tests
+        oidcSigAlgShouldMatchKeyType => sub {
+            for my $key (
+                qw(oidcRPMetaDataOptionsIDTokenSignAlg oidcRPMetaDataOptionsAccessTokenSignAlg oidcRPMetaDataOptionsUserInfoSignAlg)
+              )
+            {
+                foreach my $rp ( keys %{ $conf->{oidcRPMetaDataOptions} } ) {
+                    return ( 0,
+"Signature algorithm shouldn't be ES* if key type is RSA ($rp/$key)"
+                      )
+                      if $conf->{oidcRPMetaDataOptions}->{$rp}->{$key}
+                      and $conf->{oidcRPMetaDataOptions}->{$rp}->{$key} =~ /^E/
+                      and $conf->{oidcServiceKeyTypeSig} ne 'EC';
+                    return ( 0,
+"Signature algorithm shouldn't be RS* or PS* if key type is EC ($rp/$key)"
+                      )
+                      if $conf->{oidcRPMetaDataOptions}->{$rp}->{$key}
+                      and $conf->{oidcRPMetaDataOptions}->{$rp}->{$key} !~ /^E/
+                      and $conf->{oidcServiceKeyTypeSig} eq 'EC';
+                }
+            }
+            return 1;
+        },
+
+# Warn if both oidcRPMetaDataOptionsJwks and oidcRPMetaDataOptionsJwksUri is set
+        noJwksDuplication => sub {
+            return 1
+              unless $conf->{oidcRPMetaDataOptions}
+              and ref $conf->{oidcRPMetaDataOptions};
+            my @pb;
+            for my $rp ( keys %{ $conf->{oidcRPMetaDataOptions} } ) {
+                push @pb, $rp
+                  if $conf->{oidcRPMetaDataOptions}->{$rp}
+                  ->{oidcRPMetaDataOptionsJwks}
+                  and $conf->{oidcRPMetaDataOptions}->{$rp}
+                  ->{oidcRPMetaDataOptionsJwksUri};
+            }
+            return 1 unless @pb;
+            return ( 1,
+                "JWKS URI defined while JWKS document is fixed: "
+                  . join( ', ', @pb ) );
+        },
     };
 }
 
