@@ -14,7 +14,7 @@ use Lemonldap::NG::Portal::Main::Constants qw(
   PE_SAML_DESTINATION_ERROR
   PE_SAML_SESSION_ERROR
   PE_SAML_SIGNATURE_ERROR
-  PE_SAML_SLO_ERROR
+  PE_SLO_ERROR
   PE_SAML_SSO_ERROR
   PE_ISSUERMISSINGREQATTR
   PE_UNKNOWNPARTNER
@@ -22,7 +22,7 @@ use Lemonldap::NG::Portal::Main::Constants qw(
   PE_UNAUTHORIZEDPARTNER
 );
 
-our $VERSION = '2.0.14';
+our $VERSION = '2.18.0';
 
 extends 'Lemonldap::NG::Portal::Main::Issuer',
   'Lemonldap::NG::Portal::Lib::SAML';
@@ -405,6 +405,21 @@ sub run {
 
                     return PE_UNKNOWNPARTNER;
                 }
+                elsif ( $lasso_error ==
+                    Lasso::Constants::PROFILE_ERROR_INVALID_PROTOCOLPROFILE )
+                {
+                    $self->logger->error(
+                        "SSO: Fail to process authentication request");
+                    if ( my $ACS =
+                        eval { $login->request->AssertionConsumerServiceURL } )
+                    {
+                        $self->logger->error(
+                                "Requested AssertionConsumerServiceURL $ACS"
+                              . " might be missing from registered metadata" );
+                    }
+
+                    return PE_SAML_SSO_ERROR;
+                }
                 else {
                     $self->logger->error(
                         "SSO: Fail to process authentication request");
@@ -509,9 +524,21 @@ sub run {
                 or $nameIDFormat eq $self->getNameIDFormat("unspecified") )
             {
                 $nameIDFormat = $self->getNameIDFormat($nameIDFormatKey);
+
+                # Update NameIDFormat in request
                 eval {
+                    if ( !$login->request()->NameIDPolicy ) {
+
+                        # Create NameIDFormat object if it doesnt exist
+                        $login->request()
+                          ->NameIDPolicy( Lasso::Samlp2NameIDPolicy->new() );
+                    }
                     $login->request()->NameIDPolicy()->Format($nameIDFormat);
                 };
+                if ($@) {
+                    $self->logger->warn(
+                        "Could not update NameIDPolicy in request: $@");
+                }
             }
 
             # Force AllowCreate to TRUE for transient/persistent NameIDPolicy
@@ -1389,7 +1416,7 @@ sub logout {
     if ($session) {
         unless ( $self->setSessionFromDump( $logout, $session ) ) {
             $self->logger->error("Unable to load Lasso Session");
-            return PE_SAML_SLO_ERROR;
+            return PE_SLO_ERROR;
         }
         $self->logger->debug("Lasso Session loaded");
     }
@@ -1404,7 +1431,7 @@ sub logout {
     if ($identity) {
         unless ( $self->setIdentityFromDump( $logout, $identity ) ) {
             $self->logger->error("Unable to load Lasso Identity");
-            return PE_SAML_SLO_ERROR;
+            return PE_SLO_ERROR;
         }
         $self->logger->debug("Lasso Identity loaded");
     }
@@ -1572,7 +1599,7 @@ sub sloRelayTerm {
 
     unless ($logout_dump) {
         $self->logger->error("Could not get logout dump");
-        return PE_SAML_SLO_ERROR;
+        return PE_SLO_ERROR;
     }
 
     # Rebuild Lasso::Logout object
@@ -1580,18 +1607,18 @@ sub sloRelayTerm {
 
     unless ($logout) {
         $self->logger->error("Could not build Lasso::Logout");
-        return PE_SAML_SLO_ERROR;
+        return PE_SLO_ERROR;
     }
 
     # Inject session
     unless ($session_dump) {
         $self->logger->error("Could not get session dump");
-        return PE_SAML_SLO_ERROR;
+        return PE_SLO_ERROR;
     }
 
     unless ( $self->setSessionFromDump( $logout, $session_dump ) ) {
         $self->logger->error("Could not set session from dump");
-        return PE_SAML_SLO_ERROR;
+        return PE_SLO_ERROR;
     }
 
     # Get Lasso::Session
@@ -1599,7 +1626,7 @@ sub sloRelayTerm {
 
     unless ($session) {
         $self->logger->error("Could not get session from logout");
-        return PE_SAML_SLO_ERROR;
+        return PE_SLO_ERROR;
     }
 
     # Loop on assertions and remove them if SLO status is OK
@@ -1644,7 +1671,7 @@ sub sloRelayTerm {
     }
     else {
         $self->logger->error("Fail to send SLO response");
-        return PE_SAML_SLO_ERROR;
+        return PE_SLO_ERROR;
     }
 
 }
@@ -1662,14 +1689,14 @@ sub sloResume {
 
     unless ($ResumeParams) {
         $self->logger->error("Could not find resumption info");
-        return PE_SAML_SLO_ERROR;
+        return PE_SLO_ERROR;
     }
 
     my $logoutContextSession = $self->getSamlSession($ResumeParams);
 
     unless ($logoutContextSession) {
         $self->logger->error("Could not find logout context session");
-        return PE_SAML_SLO_ERROR;
+        return PE_SLO_ERROR;
     }
 
     my $spConfKey   = $logoutContextSession->data->{spConfKey};
@@ -2323,32 +2350,25 @@ sub attributeServer {
     ];
 }
 
-# INTERNAL METHODS
+# LEGACY METHODS
+# Remove in 3.0
 
 sub imgok {
-    my ( $self, $req, ) = @_;
-    return $self->sendImage( $req, 'icons/ok.png' );
+    my $self = shift;
+    return $self->p->imgok(@_);
 }
 
 sub imgnok {
-    my ( $self, $req, ) = @_;
-    return $self->sendImage( $req, 'icons/warning.png' );
+    my $self = shift;
+    return $self->p->imgnok(@_);
 }
 
 sub sendImage {
-    my ( $self, $req,, $img ) = @_;
-    return [
-        302,
-        [
-                'Location' => $self->conf->{portal}
-              . $self->p->staticPrefix
-              . '/common/'
-              . $img,
-        ],
-        [],
-    ];
+    my $self = shift;
+    return $self->p->sendImage(@_);
 }
 
+# INTERNAL METHODS
 # Normalize url to be tolerant to SAML Path
 # Usefull if SAML Path is a regex
 # @return normalized url
