@@ -2,7 +2,7 @@
 # into "plugins" list in lemonldap-ng.ini, section "portal"
 package Lemonldap::NG::Portal::Main::Plugins;
 
-our $VERSION = '2.0.11';
+our $VERSION = '2.18.0';
 
 package Lemonldap::NG::Portal::Main;
 
@@ -19,6 +19,7 @@ our @pList = (
     portalStatus                        => '::Plugins::Status',
     cda                                 => '::Plugins::CDA',
     notification                        => '::Plugins::Notifications',
+    rememberAuthChoiceRule              => '::Plugins::RememberAuthChoice',
     stayConnected                       => '::Plugins::StayConnected',
     portalCheckLogins                   => '::Plugins::History',
     bruteForceProtection                => '::Plugins::BruteForceProtection',
@@ -28,14 +29,22 @@ our @pList = (
     checkState                          => '::Plugins::CheckState',
     portalForceAuthn                    => '::Plugins::ForceAuthn',
     checkUser                           => '::Plugins::CheckUser',
-    impersonationRule                   => '::Plugins::Impersonation',
+    checkDevOps                         => '::Plugins::CheckDevOps',
     contextSwitchingRule                => '::Plugins::ContextSwitching',
     decryptValueRule                    => '::Plugins::DecryptValue',
     findUser                            => '::Plugins::FindUser',
-    adaptativeAuthenticationLevelRules =>
+    newLocationWarning                  => '::Plugins::NewLocationWarning',
+    passwordPolicyActivation            => '::Plugins::BasePasswordPolicy',
+    checkHIBP                           => '::Plugins::CheckHIBP',
+    checkEntropy                        => '::Plugins::CheckEntropy',
+    initializePasswordReset             => '::Plugins::InitializePasswordReset',
+    adaptativeAuthenticationLevelRules  =>
       '::Plugins::AdaptativeAuthenticationLevel',
-    globalLogoutRule => '::Plugins::GlobalLogout',
-    refreshSessions  => '::Plugins::Refresh',
+    refreshSessions     => '::Plugins::Refresh',
+    crowdsec            => '::Plugins::CrowdSec',
+    locationDetect      => '::Plugins::LocationDetect',
+    globalLogoutRule    => '::Plugins::GlobalLogout',
+    samlFederationFiles => '::Plugins::SamlFederation',
 );
 
 ##@method list enabledPlugins
@@ -79,11 +88,12 @@ sub enabledPlugins {
       if ( $conf->{soapSessionServer}
         or $conf->{soapConfigServer} );
 
-    # Add REST (check is done by it)
+    # Add REST (check is done by plugin itself)
     push @res, '::Plugins::RESTServer';
 
+    # Check if password is enabled
     if ( my $p = $conf->{passwordDB} ) {
-        push @res, "::Password::$p" if ( $p ne 'Null' );
+        push @res, "::Password::$p";
     }
 
     # Check if register is enabled
@@ -91,11 +101,18 @@ sub enabledPlugins {
       if ( $conf->{registerDB} and $conf->{registerDB} ne 'Null' );
 
     # Check if custom plugins are required
-    # TODO: change this name
     if ( $conf->{customPlugins} ) {
         $self->logger->debug( 'Custom plugins: ' . $conf->{customPlugins} );
-        push @res, grep ( /\w+/, split( /,\s*/, $conf->{customPlugins} ) );
+        push @res, grep ( /\w+/, split( /[,\s]+/, $conf->{customPlugins} ) );
     }
+
+    # Impersonation overwrites req->step and pops 'afterData' EP.
+    # Static and custom 'afterData' plugins will be never launched
+    # if they are loaded after Impersonation.
+    # This plugin must be the last 'afterData' loaded plugin. Fix #2655
+    push @res, '::Plugins::Impersonation'
+      if $conf->{impersonationRule};
+
     return @res;
 }
 
