@@ -3,15 +3,18 @@ package Lemonldap::NG::Portal::UserDB::OpenIDConnect;
 use strict;
 use Mouse;
 use Lemonldap::NG::Portal::Main::Constants qw(
+  PE_OIDC_AUTH_ERROR
   PE_BADCREDENTIALS
   PE_ERROR
   PE_OK
 );
 
-our $VERSION = '2.0.11';
+our $VERSION = '2.0.15';
 
-extends 'Lemonldap::NG::Common::Module',
-  'Lemonldap::NG::Portal::Lib::OpenIDConnect';
+extends qw(
+  Lemonldap::NG::Common::Module
+  Lemonldap::NG::Portal::Lib::OpenIDConnect
+);
 
 # INITIALIZATION
 
@@ -24,7 +27,8 @@ sub init {
 
 sub getUser {
     my ( $self, $req ) = @_;
-    my $op = $req->data->{_oidcOPCurrent};
+
+    my ( $op, $access_token ) = $self->getUserInfoParams($req);
 
     # This is likely to happen when running getUser without extractFormInfo
     # see #1980
@@ -33,7 +37,10 @@ sub getUser {
         return PE_ERROR;
     }
 
-    my $access_token = $req->data->{access_token};
+    unless ($access_token) {
+        $self->logger->warn("Could not get Access Token for User Info request");
+        return PE_ERROR;
+    }
 
     my $userinfo_content = $self->getUserInfo( $op, $access_token );
 
@@ -42,14 +49,21 @@ sub getUser {
         return PE_OK;
     }
 
-    $self->logger->debug("UserInfo received: $userinfo_content");
+    # call oidcGotUserInfo hook
+    my $h =
+      $self->p->processHook( $req, 'oidcGotUserInfo', $op, $userinfo_content, );
+    return PE_OIDC_AUTH_ERROR if ( $h != PE_OK );
 
-    $req->data->{OpenIDConnect_user_info} =
-      $self->decodeJSON($userinfo_content);
+    $req->data->{OpenIDConnect_user_info} = $userinfo_content;
 
     # Check that received sub is the same than current user
-    unless ( $req->data->{OpenIDConnect_user_info}->{sub} eq $req->{user} ) {
-        $self->logger->error("Received sub do not match current user");
+    my $id_token_sub =
+      $req->data->{id_token_sub} || $req->userData->{_oidc_sub};
+    my $received_sub = $req->data->{OpenIDConnect_user_info}->{'sub'};
+    unless ( $received_sub eq $id_token_sub ) {
+        $self->logger->error(
+"Received sub $received_sub does not match current user $id_token_sub"
+        );
         return PE_BADCREDENTIALS;
     }
 
@@ -59,7 +73,7 @@ sub getUser {
 sub findUser {
 
     # Nothing to do here
-    PE_OK;
+    return PE_OK;
 }
 
 # Get all required attributes
@@ -69,19 +83,19 @@ sub setSessionInfo {
 
     my %vars = (
         %{ $self->conf->{exportedVars} },
-        %{ $self->conf->{oidcOPMetaDataExportedVars}->{$op} }
+        %{ $self->opAttributes->{$op} }
     );
 
     while ( my ( $k, $v ) = each %vars ) {
         $req->{sessionInfo}->{$k} = $req->data->{OpenIDConnect_user_info}->{$v};
     }
 
-    PE_OK;
+    return PE_OK;
 }
 
 # Does nothing
 sub setGroups {
-    PE_OK;
+    return PE_OK;
 }
 
 1;
